@@ -16,7 +16,7 @@ from . import cosem_interface_class as ic
 from . import events as e_
 from .activity_calendar import ActivityCalendar
 from .arbitrator import Arbitrator
-from .association_ln.ver0 import AssociationLN as AssociationLNVer0
+from .association_ln.ver0 import AssociationLN as AssociationLNVer0, ServerSAP
 from .association_ln.ver1 import AssociationLN as AssociationLNVer1
 from .association_ln.ver2 import AssociationLN as AssociationLNVer2
 from .push_setup.ver2 import PushSetup as PushSetupVer2
@@ -48,7 +48,9 @@ import xml.etree.ElementTree as ET
 from ..relation_to_OBIS import get_name
 from ..cosem_interface_classes import implementations as impl, class_id as c_id
 from .. import settings
-
+from ..enums import TagsName
+from . import obis as o
+from .. import pdu_enums as pdu
 
 match settings.get_current_language():
     case settings.Language.ENGLISH:        from ..Values.EN import actors
@@ -157,8 +159,8 @@ class Collection:
         """ append objects from xml file """
         tree = ET.parse(filename)
         objects = tree.getroot()
-        if use is None and objects.tag != "Configure":
-            raise ValueError(F"ERROR: Root tag got {objects.tag}, expected 'Configure'")
+        if use is None and objects.tag != TagsName.DEVICE_ROOT.value:
+            raise ValueError(F"ERROR: Root tag got {objects.tag}, expected {TagsName.DEVICE_ROOT.value}")
         root_version: AppVersion = AppVersion.from_str(objects.attrib.get('version', '1.0.0'))
         logger.info(F'Версия: {root_version}, file: {filename.split("/")[-1]}')
         match root_version:
@@ -367,8 +369,11 @@ class Collection:
                 return ProfileGeneric(ln)
             case c_id.DATA,                                              0, cst.LogicalName(0, b, 96, 1, e) if 0 <= e <= 10:
                 return Data(ln)
-            case c_id.DATA,                                              0, cst.LogicalName(0, b, 96, 2, e):
+            # 6.2.44 Parameter changes and calibration objects
+            case c_id.DATA,                                              0, cst.LogicalName(0, b, 96, 2, 0 | 4 | 10 | 13):
                 return Data(ln)
+            case c_id.DATA,                                              0, cst.LogicalName(0, b, 96, 2, 1 | 2 | 3 | 5 | 6 | 7 | 11 | 12):
+                return impl.data.AnyDateTime(ln)
             case c_id.DATA,                                              0, cst.LogicalName(0, b, 96, 3, e) if 0 <= e <= 4:
                 return Data(ln)
             case c_id.DISCONNECT_CONTROL,                                0, cst.LogicalName(0, b, 96, 3, 10):
@@ -388,8 +393,12 @@ class Collection:
                 return Arbitrator(ln)
             case c_id.DATA,                                              0, cst.LogicalName(0, b, 96, 4, e) if 0 <= e <= 4:
                 return Data(ln)
-            case c_id.DATA,                                              0, cst.LogicalName(0, b, 96, 5, e) if 0 <= e <= 4:
+            case c_id.DATA | c_id.PROFILE_GENERIC as i,                  v, cst.LogicalName(0, b, 96, 5, 0):  # TODO: add RegisterTable
+                return get_type_from_class(i, v)(ln)
+            case c_id.DATA,                                              0, cst.LogicalName(0, b, 96, 5, e) if 1 <= e <= 4:  # TODO: add StatusMaping
                 return Data(ln)
+            case c_id.DATA,                                              0, cst.LogicalName(0, 0, 96, 5, 132):
+                return impl.data.Unsigned(ln)  # TODO: make according with СПОДЭС3 13.9. Контроль чередования фаз
             case c_id.DATA | c_id.REGISTER | c_id.EXT_REGISTER as i,     0, cst.LogicalName(0, b, 96, 8, e) if 0 <= e <= 63:
                 return get_type_from_class(i, 0)(ln)
             case c_id.REGISTER | c_id.EXT_REGISTER as i,                 0, cst.LogicalName(0, b, 96, 9, e) if 0 <= e <= 2:
@@ -432,10 +441,17 @@ class Collection:
                 ret = get_type_from_class(i, 0)(ln)
                 ret.events = e_.reactive_power_events
                 return ret
-            case c_id.DATA | c_id.REGISTER | c_id.EXT_REGISTER as i,     0, cst.LogicalName(0, b, 96, 12, e) if 0 <= e <= 6:
+            case c_id.DATA,                                              0, cst.LogicalName(0, b, 96, 12, 4):
+                return impl.data.CommunicationPortParameter(ln)
+            case c_id.DATA | c_id.REGISTER | c_id.EXT_REGISTER as i,     0, cst.LogicalName(0, b, 96, 12, e) if e in (0, 1, 2, 3, 5, 6):
                 return get_type_from_class(i, 0)(ln)
             case c_id.DATA,                                              0, cst.LogicalName(0, b, 96, 12, 128):
                 return Data(ln)
+            # 6.2.57 Consumer message objects
+            case c_id.DATA,                                              0, cst.LogicalName(0, 128, 96, 13, 1):
+                return impl.data.ITEBitMap(ln)
+            case c_id.DATA | c_id.REGISTER | c_id.EXT_REGISTER as i,     0, cst.LogicalName(0, b, 96, 13, 0 | 1):
+                return get_type_from_class(i, 0)(ln)
             case c_id.DATA | c_id.REGISTER | c_id.EXT_REGISTER as i,     0, cst.LogicalName(0, b, 96, 15, e) if 0 <= e <= 99:
                 return get_type_from_class(i, 0)(ln)
             case c_id.DATA | c_id.REGISTER | c_id.EXT_REGISTER as i,     0, cst.LogicalName(0, b, 96, 20, e):
@@ -599,8 +615,13 @@ class Collection:
                 self.__container.remove(obj)
                 return True
             case ic.COSEMInterfaceClasses() as obj, list():
-                obj.attributes.remove_by(indexes)
-                if len(obj.attributes) <= 1:
+                obj: InterfaceClass
+                for i in indexes:
+                    obj.clear_attr(i)
+                for i in range(2, obj.get_attr_length()):
+                    if obj.get_attr(i) is not None:
+                        break
+                else:
                     self.__container.remove(obj)  # TODO: Rewrite with new list attr API
                 return True
             case _:
@@ -709,14 +730,26 @@ class Collection:
         else:
             return obj
 
+    @cached_property
+    def LDN(self) -> Data:
+        return self.__get_object(o.LDN)
+
     @cached_property
     def current_association(self) -> AssociationLN:
-        return self.__get_object(bytes((0, 0, 40, 0, 0, 255)))
+        return self.__get_object(o.CURRENT_ASSOCIATION)
+
+    lru_cache(4)
+    def getASSOCIATION(self, instance: int) -> AssociationLN:
+        return self.__get_object(bytes((0, 0, 40, 0, instance, 255)))
 
     @cached_property
     def PUBLIC_ASSOCIATION(self) -> AssociationLN:
         return self.__get_object(bytes((0, 0, 40, 0, 1, 255)))
 
+    @cached_property
+    def COMMUNICATION_PORT_PARAMETER(self) -> impl.data.CommunicationPortParameter:
+        return self.__get_object(bytes((0, 0, 96, 12, 4, 255)))
+
     @cached_property
     def client_setup(self) -> ClientSetup:
         return self.__get_object(bytes((0, 0, 199, 255, 255, 255)))
@@ -892,3 +925,69 @@ class Collection:
                 raise ValueError(F"not find {selector} in {obj}")
         else:
             raise ValueError(F"object with {ln} is not {ScriptTable.NAME}")
+
+    def get_association_id(self, sap: ServerSAP) -> int:
+        """return id(association instance) from it server address"""
+        for ass in self.get_objects_by_class_id(AssociationLNVer0.CLASS_ID):
+            if ass.associated_partners_id.server_SAP == sap:
+                return ass.logical_name.e
+            else:
+                continue
+        else:
+            raise ValueError(F"absent association with {sap}")
+
+    @lru_cache(maxsize=1000)
+    def is_writable(self, ln: cst.LogicalName,
+                    index: int,
+                    association_id: int,
+                    security_policy: pdu.SecurityPolicy = pdu.SecurityPolicyVer0.NOTHING
+                    ) -> bool:
+        match self.getASSOCIATION(association_id).object_list.get_attr_access(ln, index):
+            case pdu.AttributeAccess.NO_ACCESS | pdu.AttributeAccess.READ_ONLY | pdu.AttributeAccess.AUTHENTICATED_READ_ONLY:
+                return False
+            case pdu.AttributeAccess.WRITE_ONLY | pdu.AttributeAccess.READ_AND_WRITE:
+                return True
+            case pdu.AttributeAccess.AUTHENTICATED_WRITE_ONLY | pdu.AttributeAccess.AUTHENTICATED_READ_AND_WRITE:
+                if isinstance(security_policy, pdu.SecurityPolicyVer0):
+                    match security_policy:
+                        case pdu.SecurityPolicyVer0.AUTHENTICATED | pdu.SecurityPolicyVer0.AUTHENTICATED_AND_ENCRYPTED:
+                            return True
+                        case _:
+                            return False
+                elif isinstance(security_policy, pdu.SecurityPolicyVer1):
+                    if bool(security_policy & (pdu.SecurityPolicyVer1.AUTHENTICATED_REQUEST | pdu.SecurityPolicyVer1.AUTHENTICATED_RESPONSE)):
+                        return True
+                    else:
+                        return False
+                else:
+                    raise TypeError(F"unknown {security_policy.__class__}: {security_policy}")
+            case err:
+                raise exc.ITEApplication(F"unsupport access: {err}")
+
+    @lru_cache(maxsize=1000)
+    def is_readable(self, ln: cst.LogicalName,
+                    index: int,
+                    association_id: int,
+                    security_policy: pdu.SecurityPolicy = pdu.SecurityPolicyVer0.NOTHING
+                    ) -> bool:
+        match self.getASSOCIATION(association_id).object_list.get_attr_access(ln, index):
+            case pdu.AttributeAccess.NO_ACCESS | pdu.AttributeAccess.WRITE_ONLY | pdu.AttributeAccess.AUTHENTICATED_WRITE_ONLY:
+                return False
+            case pdu.AttributeAccess.READ_ONLY | pdu.AttributeAccess.READ_AND_WRITE:
+                return True
+            case pdu.AttributeAccess.AUTHENTICATED_READ_ONLY | pdu.AttributeAccess.AUTHENTICATED_READ_AND_WRITE:
+                if isinstance(security_policy, pdu.SecurityPolicyVer0):
+                    match security_policy:
+                        case pdu.SecurityPolicyVer0.AUTHENTICATED | pdu.SecurityPolicyVer0.AUTHENTICATED_AND_ENCRYPTED:
+                            return True
+                        case _:
+                            return False
+                elif isinstance(security_policy, pdu.SecurityPolicyVer1):
+                    if bool(security_policy & (pdu.SecurityPolicyVer1.AUTHENTICATED_REQUEST | pdu.SecurityPolicyVer1.AUTHENTICATED_RESPONSE)):
+                        return True
+                    else:
+                        return False
+                else:
+                    raise TypeError(F"unknown {security_policy.__class__}: {security_policy}")
+            case err:
+                raise exc.ITEApplication(F"unsupport access: {err}")
