@@ -144,13 +144,13 @@ def handle_B(value: int) -> str:
     if value == 0:
         return ""
     elif value <= 64:
-        F", {rn.CHANNEL} {value}"
+        return F", {rn.CHANNEL} {value}"
     elif value <= 127:
-        F", {rn.UTILITY_SPECIFIC} {value}"
+        return F", {rn.UTILITY_SPECIFIC} {value}"
     elif value <= 199:
-        F", {rn.MANUFACTURER_SPECIFIC} {value}"
+        return F", {rn.MANUFACTURER_SPECIFIC} {value}"
     elif value <= 255:
-        F", {rn.RESERVED} {value}"
+        return F", {rn.RESERVED} {value}"
     else:
         raise ValueError(F"got group B: {value}, expect 0..255")
 
@@ -159,9 +159,9 @@ def handle_E(value: int) -> str:
     if value == 0:
         return ""
     elif value <= 127:
-        F", {rn.INSTANCE} {value}"
+        return F", {rn.INSTANCE} {value}"
     elif value <= 255:
-        F", {rn.MANUFACTURER_SPECIFIC} {value}"
+        return F", {rn.MANUFACTURER_SPECIFIC} {value}"
     else:
         raise ValueError(F"got group E: {value}, expect 0..255")
 
@@ -232,17 +232,32 @@ def get_name(logical_name: cst.LogicalName) -> str:
         case cst.LogicalName(0, b, 96, 1, 5):   return F"{rn.RU_REMOTE_CONSOLE_SERIAL_NUMBER}{handle_B(b)}"
         case cst.LogicalName(0, b, 96, 1, 6):   return F"{rn.RU_SPODES_VERSION}{handle_B(b)}"
         case cst.LogicalName(0, b, 96, 1, 7):   return F"{rn.RU_DEVICE_CONNECTION_SCHEME}{handle_B(b)}"
-        case cst.LogicalName(0, b, 96, 1, 8):   return F"{rn.RU_DEVICE_NOT_METROLOGICAL_VERSION}{handle_B(b)}"
+        case cst.LogicalName(0, b, 96, 1, 8):   return F"{rn.SPODES3_NOT_METROLOGICAL_FIRMWARE_ID}{handle_B(b)}"
         case cst.LogicalName(0, b, 96, 1, 9):   return F"{rn.RU_DEVICE_ID}{handle_B(b)}"
         case cst.LogicalName(0, b, 96, 1, 10):  return F"{rn.RU_COUNTER_POINT_DATA}{handle_B(b)}"
-        case cst.LogicalName(0, b, 96, 2, 0):   return F"{rn.PARAMETER_CHANGES_AND_CALIBRATION_0}{handle_B(b)}"
+        case cst.LogicalName(0, b, 96, 2, 0):   return F"{rn.NUMBER_OF_CONFIGURATION_PROGRAM_CHANGES}{handle_B(b)}"
+        case cst.LogicalName(0, b, 96, 2, 1):   return F"{rn.DATE_A_OF_LAST_CONFIGURATION_PROGRAM_CHANGE}{handle_B(b)}"
+        case cst.LogicalName(0, b, 96, 2, 2):   return F"{rn.DATE_A_OF_LAST_TIME_SWITCH_PROGRAM_CHANGE}{handle_B(b)}"
+        case cst.LogicalName(0, b, 96, 2, 3):   return F"{rn.DATE_A_OF_LAST_RIPPLE_CONTROL_RECEIVER_PROGRAM_CHANGE}{handle_B(b)}"
+        case cst.LogicalName(0, b, 96, 2, 4):   return F"{rn.STATUS_OF_SECURITY_SWITCHES}{handle_B(b)}"
+        case cst.LogicalName(0, b, 96, 2, 5):   return F"{rn.DATE_A_OF_LAST_CALIBRATION}{handle_B(b)}"
+        case cst.LogicalName(0, b, 96, 2, 6):   return F"{rn.DATE_A_OF_NEXT_CONFIGURATION_PROGRAM_CHANGE}{handle_B(b)}"
+        case cst.LogicalName(0, b, 96, 2, 7):   return F"{rn.DATE_A_OF_ACTIVATION_OF_THE_PASSIVE_CALENDAR}{handle_B(b)}"
+        case cst.LogicalName(0, b, 96, 2, 10):   return F"{rn.NUMBER_OF_PROTECTED_CONFIGURATION_PROGRAM_CHANGES}{handle_B(b)}"
+        case cst.LogicalName(0, b, 96, 2, 11):   return F"{rn.DATE_A_OF_LAST_PROTECTED_CONFIGURATION_PROGRAM_CHANGE}{handle_B(b)}"
+        case cst.LogicalName(0, b, 96, 2, 12):   return F"{rn.DATE_A_CORRECTED_OF_LAST_CLOCK_SYNCHRONIZATION_SETTING}{handle_B(b)}"
+        case cst.LogicalName(0, b, 96, 2, 13):   return F"{rn.DATE_OF_LAST_FIRMWARE_ACTIVATION}{handle_B(b)}"
         case cst.LogicalName(0, b, 96, 3, 0):   return F"{rn.I_O_CONTROL_SIGNAL_OBJECTS_GLOBAL}{handle_B(b)}"
         case cst.LogicalName(0, b, 96, 3, 10):  return F"{cn.DISCONNECT_CONTROL}{handle_B(b)}"
         case cst.LogicalName(0, b, 96, 3, 20):  return F"{rn.RU_RELAY_LOAD_ARBITRATOR}{handle_B(b)}"
         case cst.LogicalName(0, 0, 96, 4, 1):   return rn.RU_LCD_BACKLIGHT_MODE
         case cst.LogicalName(0, 0, 96, 4, 3):   return rn.RU_LOAD_LOCK_STATUS
-        case cst.LogicalName(0, 0, 96, 5, 1):   return rn.POWER_QUALITY_STATUS_DAILY
-        case cst.LogicalName(0, 0, 96, 5, 4):   return rn.POWER_QUALITY_STATUS_QUALITY_LOG
+        case cst.LogicalName(0, 0, 96, 5, 0):   return rn.INTERNAL_OPERATING_STATUS_GLOBAL
+        case cst.LogicalName(0, 0, 96, 5, 1):   return rn.INTERNAL_OPERATING_STATUS_1
+        case cst.LogicalName(0, 0, 96, 5, 2):   return rn.INTERNAL_OPERATING_STATUS_2
+        case cst.LogicalName(0, 0, 96, 5, 3):   return rn.INTERNAL_OPERATING_STATUS_3
+        case cst.LogicalName(0, 0, 96, 5, 4):   return rn.INTERNAL_OPERATING_STATUS_4
+        case cst.LogicalName(0, 0, 96, 5, 132): return rn.SPODES3_PHASE_ALTERNATING_CONTROL
         case cst.LogicalName(0, b, 96, 8, 0):   return F"{rn.TIME_OF_OPERATION}{handle_B(b)}"
         case cst.LogicalName(0, b, 96, 8, 10):  return F"{rn.RU_DURATION_OF_FAILURE_OVERSTRAIN}{handle_B(b)}"
         case cst.LogicalName(0, b, 96, 9, 0):   return F"{rn.RU_AMBIENT_TEMPERATURE}{handle_B(b)}"
@@ -257,6 +272,7 @@ def get_name(logical_name: cst.LogicalName) -> str:
         case cst.LogicalName(0, b, 96, 11, 8):  return F"{rn.RU_EVENTS_FOR_EXCEEDING_THE_REACTIVE_POWER}{handle_B(b)}"
         case cst.LogicalName(0, b, 96, 12, 4):  return F"{rn.RU_CHANNEL_NUMBER_INTERFACE}{handle_B(b)}"
         case cst.LogicalName(0, b, 96, 12, 6):  return F"{rn.COMMUNICATION_ADDRESS}{handle_B(b)}"
+        case cst.LogicalName(0, 0, 96, 12, 128):  return F"{rn.KPZ_INTERFACE_LIST}"
         case cst.LogicalName(0, b, 96, 13, 0):  return F"{rn.LOCAL_CONSUMER_MESSAGE}{handle_B(b)}"
         case cst.LogicalName(0, b, 96, 13, 1):  return F"{rn.DISPLAY_CONSUMER_MESSAGE}{handle_B(b)}"
         case cst.LogicalName(0, b, 96, 15, 0):  return F"{rn.RU_RELAY_TRIGGERING_METER_FOR_OPENING}{handle_B(b)}"
@@ -376,7 +392,7 @@ def get_name(logical_name: cst.LogicalName) -> str:
             return F"{handle_B(b)}{rn.L2} {get_obj_names(c)} {get_processing_names(d)} {get_rate(e)}"
         case cst.LogicalName(1, b, c, d, e) if c in range(61, 81):
             return F"{handle_B(b)}{rn.L3} {get_obj_names(c)} {get_processing_names(d)} {get_rate(e)}"
-        case cst.LogicalName(1, b, 88 | 89 | 124 | 125 | 126 | 128 | 129 | 130 | 131 as c, d, e):
+        case cst.LogicalName(1, b, 88 | 89 | 124 | 125 | 126 | 128 | 129 | 130 | 131 | 133 as c, d, e):
             return F"{handle_B(b)}{get_obj_names(c)} {get_processing_names(d)} {get_rate(e)}"
         case cst.LogicalName(1, b, 91, 7, 131):   return F"{rn.RU_DIFFERENTIAL_CURRENT}. {rn.INSTANTANEOUS_VALUE}{handle_B(b)}"
         case cst.LogicalName(1, b, 91, 7, 132):   return F"{rn.RU_DIFFERENTIAL_CURRENT}. {rn.PERCENT}. {rn.INSTANTANEOUS_VALUE}{handle_B(b)}"
